<?php
#namespace Tqdev\PhpCrudApi\Controller;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Tqdev\PhpCrudApi\Cache\Cache;
use Tqdev\PhpCrudApi\Column\ReflectionService;
use Tqdev\PhpCrudApi\Controller\Responder;
use Tqdev\PhpCrudApi\Database\GenericDB;
use Tqdev\PhpCrudApi\Middleware\Router\Router;
use Tqdev\PhpCrudApi\Record\RecordService;
use Tqdev\PhpCrudApi\RequestUtils;

class HomeController {
    /**
     * @var RecordService
     */
    private $service;
    private $db;
    private $cache;
    private $responder;

    public function __construct(Router $router, Responder $responder, GenericDB $db, ReflectionService $reflection, Cache $cache)
    {
        $this->db = $db;
        $this->responder = $responder;
        $router->register('GET', '/getOnSaleProducts', array($this, 'getOnSaleProducts'));
        $router->register('GET', '/getPopularProduct', array($this, 'getPopularProduct'));
        $router->register('GET', '/getPopularCategories', array($this, 'getPopularCategories'));
    }

    public function getOnSaleProducts(ServerRequestInterface $request): ResponseInterface
    {
        global $currencies, $languages_id;

        $new_products_query = tep_db_query( "select p.products_id, pd.products_name, p.products_msrp, p.products_price, p.products_tax_class_id, p.products_image, s.specials_new_products_price, p.buy1_get1_free from " . TABLE_PRODUCTS . " p, " . TABLE_PRODUCTS_DESCRIPTION . " pd, " . TABLE_SPECIALS . " s where p.products_status = '1' and s.products_id = p.products_id and p.products_id = pd.products_id and pd.language_id = '" . (int)$languages_id . "' and s.status = '1' AND ((s.start_date = '0000-00-00' OR s.start_date < NOW()) AND (s.expires_date = '0000-00-00' OR s.expires_date > NOW())) and s.daily_deal!=1 and p.is_liquidation ='0' order by s.specials_date_added DESC limit 0,20");
	
        $deal_products_query=tep_db_query( "select p.products_id, pd.products_name, p.products_msrp, p.products_price, p.products_tax_class_id, p.products_image, s.specials_new_products_price, p.buy1_get1_free from " . TABLE_PRODUCTS . " p, " . TABLE_PRODUCTS_DESCRIPTION . " pd, " . TABLE_SPECIALS . " s where p.products_status = '1' and s.products_id = p.products_id and p.products_id = pd.products_id and pd.language_id = '" . (int)$languages_id . "' and s.status = '1' AND ((s.start_date = '0000-00-00' OR s.start_date < NOW()) AND (s.expires_date = '0000-00-00' OR s.expires_date > NOW())) and s.daily_deal=1 and p.is_liquidation ='0'");
        
        $num_new_products = tep_db_num_rows($new_products_query);
        $num_deal_products = tep_db_num_rows($deal_products_query);
        
        $product_results = array();
        if ($num_new_products > 0) {
            
            if($num_deal_products > 0) {
                $deal_products = tep_db_fetch_array($deal_products_query);
                
                $products_msrp = $currencies->display_price($deal_products['products_msrp'], tep_get_tax_rate($deal_products['products_tax_class_id']));
                
                if (tep_not_null($deal_products['specials_new_products_price'])) {
                    $products_price = $currencies->display_price($deal_products['specials_new_products_price'], tep_get_tax_rate($deal_products['products_tax_class_id']));
                } else {
                    $products_price = $currencies->display_price($deal_products['products_price'], tep_get_tax_rate($deal_products['products_tax_class_id']));
                }
                
                $products_image = 'http://www.nivensoft.com/dev/mapp/images/no_image.png';
                
                $product_results[] = array(	'products_id' 		=> $deal_products['products_id'],
                                            'products_name' 	=> $deal_products['products_name'],			
                                            'products_msrp' 	=> $products_msrp, 
                                            'products_price' 	=> $products_price, 
                                            'products_image' 	=> $products_image,
                                            'daily_deal' 		=> 1 );
            }
            
            while ($new_products = tep_db_fetch_array($new_products_query)) {
                
                $products_msrp = $currencies->display_price($new_products['products_msrp'], tep_get_tax_rate($new_products['products_tax_class_id']));
                
                if (tep_not_null($new_products['specials_new_products_price'])) {
                    $products_price = $currencies->display_price($new_products['specials_new_products_price'], tep_get_tax_rate($new_products['products_tax_class_id']));
                } else {
                    $products_price = $currencies->display_price($new_products['products_price'], tep_get_tax_rate($new_products['products_tax_class_id']));
                }
                
                $products_image = 'http://www.nivensoft.com/dev/mapp/images/no_image.png';
                
                $product_results[] = array(	'products_id' 		=> $new_products['products_id'],
                                            'products_name' 	=> $new_products['products_name'],			
                                            'products_msrp' 	=> $products_msrp, 
                                            'products_price' 	=> $products_price, 
                                            'products_image' 	=> $products_image );
            }
            
            $result = array('status' => true, 'data' => $product_results);
        } else {
            $result = array('status' => false, 'data' => 'no_records');
        }
        return $this->responder->success($result);
    }

    public function getPopularProduct(ServerRequestInterface $request): ResponseInterface
    {
        global $currencies, $languages_id;
        //$params = RequestUtils::getParams($request);
        $customer_id = (!empty($_GET['customer_id']) ? tep_db_prepare_input($_GET['customer_id']) : 0);
        $customer_group_id = (!empty($_GET['customer_group_id']) ? tep_db_prepare_input($_GET['customer_group_id']) : 0);
        if($customer_id == STORE_CUSTOMER_ID) {
            $c_store = ' and p2c.categories_id not in(415,1434)';
        } else {
            tep_get_subcategories($sd_cat, 527);
            $ex_sd_cat = array_merge($sd_cat, array(415,507,527,366,1434));
            $ex_sd_cat_str = implode(',', $ex_sd_cat);
            $c_store = " and p2c.categories_id not in($ex_sd_cat_str) and p.products_store='0'";
        }
        
        $new_products_query = tep_db_query("select p.products_id, p.products_image, p.products_tax_class_id, pd.products_name, if(s.status, s.specials_new_products_price, p.products_price) as products_price,  IF(s.status, s.specials_new_products_price, NULL) as specials_new_products_price, p.products_msrp, p.buy1_get1_free, p2c.categories_id from " . TABLE_PRODUCTS . " p left join " . TABLE_SPECIALS . " s on p.products_id = s.products_id, " . TABLE_PRODUCTS_DESCRIPTION . " pd , " . TABLE_PRODUCTS_TO_CATEGORIES . " p2c where p.products_status = '1' and p.products_id = p2c.products_id  and p.products_id = pd.products_id and pd.language_id = '" . (int)$languages_id . "' $c_store order by p.products_recent_sales DESC limit 0,15");
        
        $num_new_products = tep_db_num_rows($new_products_query);
        
        $product_results = array();
        
        if ($num_new_products > 0) {
            
            while ($new_products = tep_db_fetch_array($new_products_query)) {
                
                if(in_array($new_products['products_id'],$prids)) continue;
                $prids[] = $new_products['products_id'];

                $specials_price ='';
                if ($new_price = tep_get_products_special_price($new_products['products_id'])) $specials_price=$new_price;

                $cat_discount=0;
                if( $customer_group_id > 0) {
                    $p_cat_id = $new_products['categories_id'];
                    $cat_discount = get_cat_discount($p_cat_id,'',$new_products['products_id']);
                    if($cat_discount != 0) $new_products['products_price'] = ($new_products['products_price'] - ($new_products['products_price'] * $cat_discount));
                }
                
                
                $products_msrp = $currencies->display_price($new_products['products_msrp'], tep_get_tax_rate($new_products['products_tax_class_id']));
                
                if (tep_not_null($specials_price)) {
                    $products_price = $currencies->display_price($specials_price, tep_get_tax_rate($new_products['products_tax_class_id']));
                } else {
                    $products_price = $currencies->display_price($new_products['products_price'], tep_get_tax_rate($new_products['products_tax_class_id']));
                }

                $products_image = 'http://www.nivensoft.com/dev/mapp/images/no_image.png';
                
                $product_results[] = array(	'products_id' 		=> $new_products['products_id'],
                                            'products_name' 	=> $new_products['products_name'],			
                                            'products_msrp' 	=> $products_msrp, 
                                            'products_price' 	=> $products_price, 
                                            'products_image' 	=> $products_image,
                                            'buy1_get1_free'	=> $new_products['buy1_get1_free']);
            }
            
            $result = array('status' => true, 'data' => $product_results);
        } else {
            $result = array('status' => false, 'data' => 'no_records');
        }
        return $this->responder->success($result);
    }    
}
?>